/**
 * Copyright (C) 2008 Stuart McCulloch
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ops4j.peaberry.util.ldap;

import java.util.Map;

import org.ops4j.peaberry.AttributeFilter;
import org.ops4j.peaberry.osgi.AttributeDictionary;
import org.osgi.framework.Filter;
import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.InvalidSyntaxException;

/**
 * Implementation of LDAP {@link AttributeFilter}, uses code from Apache Felix.
 * 
 * @author mcculls@gmail.com (Stuart McCulloch)
 */
public final class LdapAttributeFilter
    implements AttributeFilter {

  private final Filter filter;

  public LdapAttributeFilter(final String ldapFilter) {
    try {
      filter = FrameworkUtil.createFilter(ldapFilter);
    } catch (final InvalidSyntaxException e) {
      throw new IllegalArgumentException("Bad LDAP filter: " + ldapFilter);
    }
  }

  public boolean matches(final Map<String, ?> attributes) {
    return filter.match(null == attributes ? null : new AttributeDictionary(attributes));
  }

  @Override
  public String toString() {
    return filter.toString();
  }

  @Override
  public int hashCode() {
    return filter.hashCode();
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj instanceof LdapAttributeFilter) {
      return filter.equals(((LdapAttributeFilter) obj).filter);
    }
    return false;
  }
}
